static const char *RcsId = "$Header: /home/cvsadm/cvsroot/fermi/servers/8smc3/src/Standa8SMC3.cpp,v 1.7 2017-09-21 10:23:10 graziano Exp $";
//+=============================================================================
//
// file :         Standa8SMC3.cpp
//
// description :  C++ source for the Standa8SMC3 and its commands. 
//                The class is derived from Device. It represents the
//                CORBA servant object which will be accessed from the
//                network. All commands which can be executed on the
//                Standa8SMC3 are implemented in this file.
//
// project :      TANGO Device Server
//
// $Author: graziano $
//
// $Revision: 1.7 $
//
// $Log: Standa8SMC3.cpp,v $
// Revision 1.7  2017-09-21 10:23:10  graziano
// fixed locking of DevSerial and auto switch Motor ON
//
// Revision 1.6  2017-03-10 13:19:04  graziano
// tango 9
//
// Revision 1.3  2015-09-08 10:26:59  graziano
// added configurable switch polarity
//
// Revision 1.2  2014-10-03 09:33:54  graziano
// fixed homing polarity, direction, removed % 360
//
// Revision 1.1  2014-09-22 13:21:58  graziano
// first commit
//
// Revision 1.13  2013-08-23 08:58:58  graziano
// Device_4Impl
//
// Revision 1.12  2011-01-19 16:18:20  graziano
// handled rw db
//
// Revision 1.11  2009/11/18 08:33:13  graziano
// inited write value of Position_home
//
// Revision 1.10  2009/09/23 15:11:26  graziano
// added attribute Position_home
//
// Revision 1.9  2009/09/23 14:46:29  graziano
// bug fixing
//
// Revision 1.8  2009/09/23 12:28:31  graziano
// handling of limit switches to allow <0 and >360 and bug fixing in state and status
//
// Revision 1.7  2009/08/03 12:04:25  graziano
// changed speed in Tiks Hz
//
// Revision 1.6  2009/08/03 10:25:05  graziano
// changed speed in degree/s
//
// Revision 1.5  2009/08/03 09:51:48  graziano
// bug fixing
//
// Revision 1.4  2009/04/27 09:01:11  graziano
// bug fixing
//
// Revision 1.3  2009/04/15 10:14:21  graziano
// fixed sleep and homing
//
// Revision 1.2  2009/04/02 13:38:15  graziano
// removed reset to left in init
//
// Revision 1.1  2008/11/19 10:39:26  graziano
// first version
//
//
// copyleft :     European Synchrotron Radiation Facility
//                BP 220, Grenoble 38043
//                FRANCE
//
//-=============================================================================
//
//  		This file is generated by POGO
//	(Program Obviously used to Generate tango Object)
//
//         (c) - Software Engineering Group - ESRF
//=============================================================================



//===================================================================
//
//	The following table gives the correspondence
//	between commands and method name.
//
//  Command name  |  Method name
//	----------------------------------------
//  State         |  dev_state()
//  Status        |  dev_status()
//  Forward       |  forward()
//  Backward      |  backward()
//  Stop          |  stop()
//  MotorOFF      |  motor_off()
//  MotorON       |  motor_on()
//  MotorSTANDBY  |  motor_standby()
//  GoHome        |  go_home()
//  SetHome       |  set_home()
//  GoToPosition  |  go_to_position()
//  FindEdge      |  find_edge()
//
//===================================================================


#include <tango.h>
#include <Standa8SMC3.h>
#include <Standa8SMC3Class.h>
#include "standathread.h"

namespace Standa8SMC3_ns
{

//+----------------------------------------------------------------------------
//
// method : 		Standa8SMC3::Standa8SMC3(string &s)
// 
// description : 	constructor for simulated Standa8SMC3
//
// in : - cl : Pointer to the DeviceClass object
//      - s : Device name 
//
//-----------------------------------------------------------------------------
Standa8SMC3::Standa8SMC3(Tango::DeviceClass *cl,string &s)
:TANGO_BASE_CLASS(cl,s.c_str())
{
	init_device();
}

Standa8SMC3::Standa8SMC3(Tango::DeviceClass *cl,const char *s)
:TANGO_BASE_CLASS(cl,s)
{
	init_device();
}

Standa8SMC3::Standa8SMC3(Tango::DeviceClass *cl,const char *s,const char *d)
:TANGO_BASE_CLASS(cl,s,d)
{
	init_device();
}
//+----------------------------------------------------------------------------
//
// method : 		Standa8SMC3::delete_device()
// 
// description : 	will be called at device destruction or at init command.
//
//-----------------------------------------------------------------------------
void Standa8SMC3::delete_device()
{
	if(get_state()==Tango::MOVING)
		stop();
	//DEBUG_STREAM << "Standa8SMC3::delete_device() entering..."<<endl;
	abortflag = 1;
	sleep(1);
	
/*	*attr_Position_read = (position_tmp + position_moving) / (double)positionRatio;
	DEBUG_STREAM << "Standa8SMC3::delete_device() saving position="<<*attr_Position_read<<" before deleting"<<endl;
	Tango::DbDatum as("current_position");
	as << *attr_Position_read;
	Tango::DbData	data;
	data.push_back(as);
	get_db_device()->put_property(data);
*/
	Tango::Database *db = new Tango::Database(host_rw,port_rw);
	try
	{
		Tango::DbDatum as("Memo_position");
		as << memo_position;
		Tango::DbData	data;
		data.push_back(as);
		db->put_device_property(get_name(), data);
	}
	catch(Tango::DevFailed &e)
	{
		INFO_STREAM << __func__<<" error memorizing Position: "<<e.errors[0].desc;
	}
	delete db;

	//	Delete device's allocated object
	//DEBUG_STREAM << "Standa8SMC3::delete_device() before delete attr_Position_read"<<endl;
	delete attr_Position_read;
	//DEBUG_STREAM << "Standa8SMC3::delete_device() before delete attr_Acceleration_read"<<endl;
	delete attr_Acceleration_read;
	//DEBUG_STREAM << "Standa8SMC3::delete_device() before delete [] attr_Speed_read"<<endl;
	delete [] attr_Speed_read;
	//DEBUG_STREAM << "Standa8SMC3::delete_device() before delete attr_Accuracy_read"<<endl;
	delete attr_Accuracy_read;	
	//DEBUG_STREAM << "Standa8SMC3::delete_device() before delete attr_Stat_read"<<endl;
	delete attr_Stat_read;
//	delete attr_Total_steps_read;
//	delete attr_Steps_sx_read;
//	delete attr_Steps_st_read;	
	/*if(attr_Speed_write)
		delete [] attr_Speed_write;
		*/
	delete attr_Position_home_read;
	//DEBUG_STREAM << "Standa8SMC3::delete_device() before delete mutex"<<endl;
	delete mutex;
	//DEBUG_STREAM << "Standa8SMC3::delete_device() before delete devserial"<<endl;
	delete devserial;
	//DEBUG_STREAM << "Standa8SMC3::delete_device() exiting ..."<<endl;
	//Tango::leavefunc();
}

//+----------------------------------------------------------------------------
//
// method : 		Standa8SMC3::init_device()
// 
// description : 	will be called at device initialization.
//
//-----------------------------------------------------------------------------
void Standa8SMC3::init_device()
{
	INFO_STREAM << "Standa8SMC3::Standa8SMC3() create device " << device_name << endl;

	// Initialise variables to default values
	//--------------------------------------------
	abortflag = 0;	
	maxAllowedSpeed = 9091;
	limit_switch_polarity = 1;
	get_device_property();
	
	if(positionRatio > 0)
	{
		strcpy(positive_rotation_way, "1");
		strcpy(negative_rotation_way, "0");
		homing_direction = 0;
	}
	else
	{
		positionRatio = -positionRatio;
		strcpy(positive_rotation_way, "0");
		strcpy(negative_rotation_way, "1");
		homing_direction = 1;
	}
	
	position_tmp = 0;
	position_moving = 0;
	position_valid = true;
	moving = false;
	homing = false;
	
	//lettura host_rw e port_rw dal database
	Tango::Database *db1 = new Tango::Database();
	try
	{
	    Tango::DbData db_data;
	    db_data.push_back((Tango::DbDatum("Host")));
	    db_data.push_back((Tango::DbDatum("Port")));
	    db1->get_property("Database",db_data);
	 
	    db_data[0] >> host_rw;
	    db_data[1] >> port_rw;
	}catch(Tango::DevFailed &e)
	{
	    INFO_STREAM << __FUNCTION__ << " Error reading Database property='" << e.errors[0].desc << "'";
	}
	delete db1; 
	
	//set_change_event("State", true, true);		//attribute name, implemented, detect
	
	Tango::Database *db = new Tango::Database();
	Tango::DbData db_data;	
	try {
		db_data.push_back(Tango::DbDatum("timeout"));
		string in_prop("timeout");
//		devserial->get_property(in_prop, db_data);
		db->get_device_property(serialdevice,db_data);
		db_data[0] >> serial_timeout;
		delete db;
	}
	catch(Tango::DevFailed e) {
		string temp;
		temp = e.errors[0].desc;
		WARN_STREAM << "Standa8SMC3::Standa8SMC3() error getting timeout property from serial device: " << temp << endl;
		delete db;		
	}
	if(serial_timeout < 0 || serial_timeout > 2000)
		serial_timeout = 500;
	
	/* connect to serialdevice */
	devserial = new Tango::DeviceProxy(serialdevice);	
	
	/* check if serialdevice is responding */
	try {
		if (devserial->ping()) 
		{
			devserial->set_timeout_millis(serial_timeout);
			set_state(Tango::UNKNOWN);	
			set_status("Connected to serialdevice " + serialdevice);
			old_state = device_state;
			old_status = device_status;
			INFO_STREAM << "Connected to serialdevice " + serialdevice << endl;
		}	
	} 
	catch(Tango::DevFailed e) {
		set_state(Tango::FAULT);
		set_status("Failed to connect to serialdevice " + serialdevice);
		old_state = device_state;
		old_status = device_status;
		INFO_STREAM << "Failed to connect to serialdevice " + serialdevice << endl; 			
	}
	
	if(!socket)
	{
		Tango::DeviceData Din;
		Tango::DevLong flush = 2;  //flush input and output
		Din << flush;
		try	{
			devserial->command_inout("DevSerFlush",Din);
		}
		catch(Tango::DevFailed e) {
			string temp;
			temp = e.errors[0].desc;
			WARN_STREAM << "Standa8SMC3::Standa8SMC3() error flushing serial device: " << temp << endl;
		}
	}
	else
	{
		Tango::DeviceData Dout;
		try {
			Dout = devserial->command_inout("Read");
			if(!Dout.is_empty())
			{
				string out;
				Dout >> out;
				DEBUG_STREAM << "Standa8SMC3::Standa8SMC3(): initially read from socket: " <<  out << endl;
			}
		}
		catch(Tango::DevFailed e) {
			TangoSys_MemStream out_stream;
			out_stream << "Read command from SocketDeviceServer failed" << ends;
			DEBUG_STREAM << "Standa8SMC3::Standa8SMC3(): " <<  out_stream.str() << endl;
		}
	}
	
	
	attr_Position_read = new Tango::DevDouble();
	attr_Acceleration_read = new Tango::DevDouble();
	attr_Speed_read = new Tango::DevDouble[2];
	attr_Accuracy_read = new Tango::DevDouble();
	attr_Stat_read = new Tango::DevLong();
	attr_Position_home_read = new Tango::DevDouble();
//	attr_Total_steps_read = new Tango::DevLong();
//	attr_Steps_sx_read = new Tango::DevLong();
//	attr_Steps_st_read = new Tango::DevLong();	
	
	//\\\\\\\\\\\\\\\\\\ CREATE ATTR THREAD  \\\\\\\\\\\\\\\\\\//
	mutex = new omni_mutex();
	readloop = new standathread(this);
	
/*	int numHi=0, numLow=0;
	for(im540_map::iterator it2 = read_data.begin(); it2 != read_data.end(); it2++)
	{
		if(it2->second.pri == Hi_pri)
			numHi++;
		else if(it2->second.pri == Low_pri)
			numLow++;			
	}
	int completeLoop = numHi*PRIORITY_FACTOR + numLow;	
	int hiPrioLoop = (int) ceil((double)(completeLoop / PRIORITY_FACTOR));
	int loPrioLoop = completeLoop;
	timeoutHiPrio = ((hiPrioLoop+1) * (serial_timeout+th_sleep+wr_sleep))+2000;
	timeoutLowPrio = ((loPrioLoop+1) * (serial_timeout+th_sleep+wr_sleep))+2000;
	INFO_STREAM << "Im540::Im540(): timeoutHiprio=" << timeoutHiPrio << " timeoutLowPrio=" << timeoutLowPrio << " serial_timeout=" << serial_timeout <<endl;
	*/
	bool retry = false;
	try {
		motor_off();
	} catch(...)
	{
		WARN_STREAM << "Standa8SMC3::init_device(): exception in motor_off!()" << endl;
		retry = true;
	}
	if(retry)
	{
		try {
			motor_off();
		} catch(...)
		{
			WARN_STREAM << "Standa8SMC3::init_device(): II exception in motor_off!()" << endl;
		}	
	}
	//############# set default accuracy ###############
	//read_Accuracy(Tango::Attribute &attr);
	string resp;
	long val = 1;
	char temp_c[32];
	sprintf(temp_c,"%d",val);
	//val = strtol(temp.str().c_str(),0,16);
	//temp >> val;
	abort_sleep((double(wr_sleep))/1000);
	if(WriteReadStanda(SET_MICRO_STEP_CONF,temp_c,resp, true))
		WARN_STREAM << "Standa8SMC3::init_device(): exception writing micro step conf: '"<< resp << "'" << endl;		
	
	abort_sleep((double(wr_sleep))/1000);
	if(WriteReadStanda(READ_MICRO_STEP_CONF,"",resp, true))
		WARN_STREAM << "Standa8SMC3::init_device(): exception reading micro step conf: '"<< resp << "'" << endl;
	
	if(resp.length() >= 4)
	{
		size_t pos = resp.find(":0");
		string temp = resp.substr(pos +2,resp.length()-pos-4);		//eliminate ':0' and /d/a
		val = strtol(temp.c_str(),0,16);
		if(val==1)
			step_factor = 1;
		else if(val==2)
			step_factor = 2;
		else if(val==3)
			step_factor = 4;
		else if(val==4)
			step_factor = 8;
	}
	attr_Accuracy_write = val;
	*attr_Accuracy_read = val;
	Tango::MultiAttribute *attr_list = get_device_attr();
	Tango::WAttribute &attr_ = attr_list->get_w_attr_by_name ("Accuracy"); 
	attr_.set_write_value(attr_Accuracy_write);	
	//############# set default speed ###############
	bool vel_err = false;
	long lval = 1;
	for(int i=1;i<256;i++)
	{
		if(defaultMinSpeed >= speeds_table[lval])
			break;
		lval = i;
		//DEBUG_STREAM << "Standa8SMC3::init_device(): MIN -	setting lval to " << lval << endl;
	}	
	DEBUG_STREAM << "Standa8SMC3::init_device(): MIN -	defaultMinSpeed=" << defaultMinSpeed << " setting lval to " << lval << endl;	
	sprintf(temp_c,"%2.2X",lval);
	abort_sleep((double(wr_sleep))/1000);
	if(WriteReadStanda(SET_MIN_VELOCITY,temp_c,resp, true))
	{
		WARN_STREAM << "Standa8SMC3::init_device(): exception writing min speed conf: '"<< resp << "'" << endl;
		vel_err = true;
	}
	lval = 1;
	for(int i=1;i<256;i++)
	{
		if(defaultMaxSpeed >= speeds_table[lval])
			break;
		lval = i;
		//DEBUG_STREAM << "Standa8SMC3::init_device(): MAX -	setting lval to " << lval << endl;
	}	
	DEBUG_STREAM << "Standa8SMC3::init_device(): MAX -	defaultMaxSpeed=" << defaultMaxSpeed << " setting lval to " << lval << endl;	
	sprintf(temp_c,"%2.2X",lval);	
	abort_sleep((double(wr_sleep))/1000);
	if(WriteReadStanda(SET_MAX_VELOCITY,temp_c,resp, true))
	{
		WARN_STREAM << "Standa8SMC3::init_device(): exception reading max speed conf: '"<< resp << "'" << endl;
		vel_err = true;
	}	
	if(!vel_err)
	{
//		Tango::WAttribute &attr_2 = attr_list->get_w_attr_by_name ("Velocity"); 
//		attr_.set_write_value(attr_Accuracy_write);		
	}	
	
	//############# set default acceleration ###############
	sprintf(temp_c,"%2.2X",defaultAcceleration);
	abort_sleep((double(wr_sleep))/1000);
	if(WriteReadStanda(SET_ACCELERATION,temp_c,resp, true))
		WARN_STREAM << "Standa8SMC3::init_device(): exception writing default acceleration: '"<< resp << "'" << endl;		
	else
	{
		attr_Acceleration_write = defaultAcceleration;
		Tango::WAttribute &attr_3 = attr_list->get_w_attr_by_name ("Acceleration"); 
		attr_3.set_write_value(attr_Acceleration_write);
	}

	//----------------- HANDLING OF LIMIT SWITCHES ---------------
	//############# enable limit switch ###############
//#if 0
	abort_sleep((double(wr_sleep))/1000);
	if(WriteReadStanda(SET_LIMIT_OFF,"1",resp, true))
		WARN_STREAM << "Standa8SMC3::init_device(): exception disabling left limit switch: '"<< resp << "'" << endl;	
	abort_sleep((double(wr_sleep))/1000);
	if(WriteReadStanda(SET_LIMIT_OFF,"2",resp, true))
		WARN_STREAM << "Standa8SMC3::init_device(): exception disabling right limit switch: '"<< resp << "'" << endl;				
//#endif

	//############# reset to left limit switch ###############
	*attr_Position_home_read = attr_Position_home_write = current_position;
	Tango::WAttribute &attr_4 = attr_list->get_w_attr_by_name ("Position_home"); 
	attr_4.set_write_value(attr_Position_home_write);
	DEBUG_STREAM << "Standa8SMC3::init_device(): initial position *attr_Position_read="<< *attr_Position_read << endl;
	
	attr_Position_write = memo_position;
	position_tmp = attr_Position_write * (double)positionRatio;
	Tango::WAttribute &attr_5 = attr_list->get_w_attr_by_name ("Position"); 
	attr_5.set_write_value(attr_Position_write);
	DEBUG_STREAM << "Standa8SMC3::init_device(): initial position ="<< attr_Position_write << endl;

	/*Tango::WAttribute &attr_4 = attr_list->get_w_attr_by_name ("Position"); 
	attr_4.get_write_value(attr_Position_write);
	*attr_Position_read = attr_Position_write;
	DEBUG_STREAM << "Standa8SMC3::init_device(): AFTER GET POSITION WRITE: initial position *attr_Position_read="<< *attr_Position_read << endl;*/	
/*	if(*attr_Position_read == 0)
	{
		abort_sleep((double(wr_sleep))/1000);
		if(WriteReadStanda(CMD_RESET,"0",resp, true))		//reset left
			WARN_STREAM << "Standa8SMC3::init_device(): exception resetting to left limit switch: '"<< resp << "'" << endl;
		set_home();		
	}
	else
	{
		position_tmp = *attr_Position_read * (double)positionRatio;		//attr_Position_read saved as current_position device property
		position_valid = true;
	}*/
	readloop->start();	
}


//+----------------------------------------------------------------------------
//
// method : 		Standa8SMC3::get_device_property()
// 
// description : 	Read the device properties from database.
//
//-----------------------------------------------------------------------------
void Standa8SMC3::get_device_property()
{
	//	Initialize your default values here (if not done with  POGO).
	//------------------------------------------------------------------

	//	Read device properties from database.(Automatic code generation)
	//------------------------------------------------------------------
	Tango::DbData	dev_prop;
	dev_prop.push_back(Tango::DbDatum("AxisNumber"));
	dev_prop.push_back(Tango::DbDatum("AxisForwardLimit"));
	dev_prop.push_back(Tango::DbDatum("AxisBackwardLimit"));
	dev_prop.push_back(Tango::DbDatum("Serialdevice"));
	dev_prop.push_back(Tango::DbDatum("Serial_timeout"));
	dev_prop.push_back(Tango::DbDatum("Wr_sleep"));
	dev_prop.push_back(Tango::DbDatum("Th_sleep"));
	dev_prop.push_back(Tango::DbDatum("PositionRatio"));
	dev_prop.push_back(Tango::DbDatum("DefaultMaxSpeed"));
	dev_prop.push_back(Tango::DbDatum("DefaultMinSpeed"));
	dev_prop.push_back(Tango::DbDatum("DefaultAcceleration"));
	dev_prop.push_back(Tango::DbDatum("MaxAllowedSpeed"));
	dev_prop.push_back(Tango::DbDatum("Current_position"));
	dev_prop.push_back(Tango::DbDatum("Limit_switch_polarity"));
	dev_prop.push_back(Tango::DbDatum("Memo_position"));
	dev_prop.push_back(Tango::DbDatum("Socket"));

	//	Call database and extract values
	//--------------------------------------------
	if (Tango::Util::instance()->_UseDb==true)
		get_db_device()->get_property(dev_prop);
	Tango::DbDatum	def_prop, cl_prop;
	Standa8SMC3Class	*ds_class =
		(static_cast<Standa8SMC3Class *>(get_device_class()));
	int	i = -1;

	//	Try to initialize AxisNumber from class property
	cl_prop = ds_class->get_class_property(dev_prop[++i].name);
	if (cl_prop.is_empty()==false)	cl_prop  >>  axisNumber;
	else {
		//	Try to initialize AxisNumber from default device value
		def_prop = ds_class->get_default_device_property(dev_prop[i].name);
		if (def_prop.is_empty()==false)	def_prop  >>  axisNumber;
	}
	//	And try to extract AxisNumber value from database
	if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  axisNumber;

	//	Try to initialize AxisForwardLimit from class property
	cl_prop = ds_class->get_class_property(dev_prop[++i].name);
	if (cl_prop.is_empty()==false)	cl_prop  >>  axisForwardLimit;
	else {
		//	Try to initialize AxisForwardLimit from default device value
		def_prop = ds_class->get_default_device_property(dev_prop[i].name);
		if (def_prop.is_empty()==false)	def_prop  >>  axisForwardLimit;
	}
	//	And try to extract AxisForwardLimit value from database
	if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  axisForwardLimit;

	//	Try to initialize AxisBackwardLimit from class property
	cl_prop = ds_class->get_class_property(dev_prop[++i].name);
	if (cl_prop.is_empty()==false)	cl_prop  >>  axisBackwardLimit;
	else {
		//	Try to initialize AxisBackwardLimit from default device value
		def_prop = ds_class->get_default_device_property(dev_prop[i].name);
		if (def_prop.is_empty()==false)	def_prop  >>  axisBackwardLimit;
	}
	//	And try to extract AxisBackwardLimit value from database
	if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  axisBackwardLimit;

	//	Try to initialize Serialdevice from class property
	cl_prop = ds_class->get_class_property(dev_prop[++i].name);
	if (cl_prop.is_empty()==false)	cl_prop  >>  serialdevice;
	else {
		//	Try to initialize Serialdevice from default device value
		def_prop = ds_class->get_default_device_property(dev_prop[i].name);
		if (def_prop.is_empty()==false)	def_prop  >>  serialdevice;
	}
	//	And try to extract Serialdevice value from database
	if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  serialdevice;

	//	Try to initialize Serial_timeout from class property
	cl_prop = ds_class->get_class_property(dev_prop[++i].name);
	if (cl_prop.is_empty()==false)	cl_prop  >>  serial_timeout;
	else {
		//	Try to initialize Serial_timeout from default device value
		def_prop = ds_class->get_default_device_property(dev_prop[i].name);
		if (def_prop.is_empty()==false)	def_prop  >>  serial_timeout;
	}
	//	And try to extract Serial_timeout value from database
	if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  serial_timeout;

	//	Try to initialize Wr_sleep from class property
	cl_prop = ds_class->get_class_property(dev_prop[++i].name);
	if (cl_prop.is_empty()==false)	cl_prop  >>  wr_sleep;
	else {
		//	Try to initialize Wr_sleep from default device value
		def_prop = ds_class->get_default_device_property(dev_prop[i].name);
		if (def_prop.is_empty()==false)	def_prop  >>  wr_sleep;
	}
	//	And try to extract Wr_sleep value from database
	if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  wr_sleep;

	//	Try to initialize Th_sleep from class property
	cl_prop = ds_class->get_class_property(dev_prop[++i].name);
	if (cl_prop.is_empty()==false)	cl_prop  >>  th_sleep;
	else {
		//	Try to initialize Th_sleep from default device value
		def_prop = ds_class->get_default_device_property(dev_prop[i].name);
		if (def_prop.is_empty()==false)	def_prop  >>  th_sleep;
	}
	//	And try to extract Th_sleep value from database
	if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  th_sleep;

	//	Try to initialize PositionRatio from class property
	cl_prop = ds_class->get_class_property(dev_prop[++i].name);
	if (cl_prop.is_empty()==false)	cl_prop  >>  positionRatio;
	else {
		//	Try to initialize PositionRatio from default device value
		def_prop = ds_class->get_default_device_property(dev_prop[i].name);
		if (def_prop.is_empty()==false)	def_prop  >>  positionRatio;
	}
	//	And try to extract PositionRatio value from database
	if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  positionRatio;

	//	Try to initialize DefaultMaxSpeed from class property
	cl_prop = ds_class->get_class_property(dev_prop[++i].name);
	if (cl_prop.is_empty()==false)	cl_prop  >>  defaultMaxSpeed;
	else {
		//	Try to initialize DefaultMaxSpeed from default device value
		def_prop = ds_class->get_default_device_property(dev_prop[i].name);
		if (def_prop.is_empty()==false)	def_prop  >>  defaultMaxSpeed;
	}
	//	And try to extract DefaultMaxSpeed value from database
	if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  defaultMaxSpeed;

	//	Try to initialize DefaultMinSpeed from class property
	cl_prop = ds_class->get_class_property(dev_prop[++i].name);
	if (cl_prop.is_empty()==false)	cl_prop  >>  defaultMinSpeed;
	else {
		//	Try to initialize DefaultMinSpeed from default device value
		def_prop = ds_class->get_default_device_property(dev_prop[i].name);
		if (def_prop.is_empty()==false)	def_prop  >>  defaultMinSpeed;
	}
	//	And try to extract DefaultMinSpeed value from database
	if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  defaultMinSpeed;

	//	Try to initialize DefaultAcceleration from class property
	cl_prop = ds_class->get_class_property(dev_prop[++i].name);
	if (cl_prop.is_empty()==false)	cl_prop  >>  defaultAcceleration;
	else {
		//	Try to initialize DefaultAcceleration from default device value
		def_prop = ds_class->get_default_device_property(dev_prop[i].name);
		if (def_prop.is_empty()==false)	def_prop  >>  defaultAcceleration;
	}
	//	And try to extract DefaultAcceleration value from database
	if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  defaultAcceleration;

	//	Try to initialize MaxAllowedSpeed from class property
	cl_prop = ds_class->get_class_property(dev_prop[++i].name);
	if (cl_prop.is_empty()==false)	cl_prop  >>  maxAllowedSpeed;
	else {
		//	Try to initialize MaxAllowedSpeed from default device value
		def_prop = ds_class->get_default_device_property(dev_prop[i].name);
		if (def_prop.is_empty()==false)	def_prop  >>  maxAllowedSpeed;
	}
	//	And try to extract MaxAllowedSpeed value from database
	if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  maxAllowedSpeed;

	//	Try to initialize Current_position from class property
	cl_prop = ds_class->get_class_property(dev_prop[++i].name);
	if (cl_prop.is_empty()==false)	cl_prop  >>  current_position;
	else {
		//	Try to initialize Current_position from default device value
		def_prop = ds_class->get_default_device_property(dev_prop[i].name);
		if (def_prop.is_empty()==false)	def_prop  >>  current_position;
	}
	//	And try to extract Current_position value from database
	if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  current_position;

	//	Try to initialize Limit_switch_polarity from class property
	cl_prop = ds_class->get_class_property(dev_prop[++i].name);
	if (cl_prop.is_empty()==false)	cl_prop  >>  limit_switch_polarity;
	else {
		//	Try to initialize Limit_switch_polarity from default device value
		def_prop = ds_class->get_default_device_property(dev_prop[i].name);
		if (def_prop.is_empty()==false)	def_prop  >>  limit_switch_polarity;
	}
	//	And try to extract Limit_switch_polarity value from database
	if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  limit_switch_polarity;

	//	Try to initialize Memo_position from class property
	cl_prop = ds_class->get_class_property(dev_prop[++i].name);
	if (cl_prop.is_empty()==false)	cl_prop  >> memo_position;
	else {
		//	Try to initialize Memo_position from default device value
		def_prop = ds_class->get_default_device_property(dev_prop[i].name);
		if (def_prop.is_empty()==false)	def_prop  >>  memo_position;
	}
	//	And try to extract Memo_position value from database
	if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  memo_position;

	//	Try to initialize Socket from class property
	cl_prop = ds_class->get_class_property(dev_prop[++i].name);
	if (cl_prop.is_empty()==false)	cl_prop  >> socket;
	else {
		//	Try to initialize Socket from default device value
		def_prop = ds_class->get_default_device_property(dev_prop[i].name);
		if (def_prop.is_empty()==false)	def_prop  >>  socket;
	}
	//	And try to extract Socket value from database
	if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  socket;



	//	End of Automatic code generation
	//------------------------------------------------------------------

}
//+----------------------------------------------------------------------------
//
// method : 		Standa8SMC3::always_executed_hook()
// 
// description : 	method always executed before any command is executed
//
//-----------------------------------------------------------------------------
void Standa8SMC3::always_executed_hook()
{
	
}
//+----------------------------------------------------------------------------
//
// method : 		Standa8SMC3::read_attr_hardware
// 
// description : 	Hardware acquisition for attributes.
//
//-----------------------------------------------------------------------------
void Standa8SMC3::read_attr_hardware(vector<long> &attr_list)
{
	//DEBUG_STREAM << "Standa8SMC3::read_attr_hardware(vector<long> &attr_list) entering... "<< endl;
	//	Add your own code here
}
//+----------------------------------------------------------------------------
//
// method : 		Standa8SMC3::read_Position_home
// 
// description : 	Extract real attribute values for Position_home acquisition result.
//
//-----------------------------------------------------------------------------
void Standa8SMC3::read_Position_home(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "Standa8SMC3::read_Position_home(Tango::Attribute &attr) entering... "<< endl;
	attr.set_value(attr_Position_home_read);
}

//+----------------------------------------------------------------------------
//
// method : 		Standa8SMC3::write_Position_home
// 
// description : 	Write Position_home attribute values to hardware.
//
//-----------------------------------------------------------------------------
void Standa8SMC3::write_Position_home(Tango::WAttribute &attr)
{
	DEBUG_STREAM << "Standa8SMC3::write_Position_home(Tango::WAttribute &attr) entering... "<< endl;
	attr.get_write_value(attr_Position_home_write);
	*attr_Position_home_read = attr_Position_home_write;
	
 //salvataggio proprietà usando host_rw e port_rw per connettersi al database
	Tango::Database *db = new Tango::Database(host_rw,port_rw);
	try {
		Tango::DbDatum as("current_position");
		as << attr_Position_home_write;
		Tango::DbData	data;
		data.push_back(as);
		db->put_device_property(get_name(), data);
	}
	catch(Tango::DevFailed &e)
	{
		INFO_STREAM << __FUNCTION__<< " error saving properties='" << e.errors[0].desc << "'";
	}
	delete db; 
}



//+----------------------------------------------------------------------------
//
// method : 		Standa8SMC3::read_Total_steps
// 
// description : 	Extract real attribute values for Total_steps acquisition result.
//
//-----------------------------------------------------------------------------
/*void Standa8SMC3::read_Total_steps(Tango::Attribute &attr)
{
	DEBUG_STREAM << "Standa8SMC3::read_Total_steps(Tango::Attribute &attr) entering... "<< endl;
	string resp;
	abort_sleep((double(wr_sleep))/1000);
	if(WriteReadStanda(READ_STEPS,"",resp, true))
		Tango::Except::throw_exception(
			(const char *) "Command failed",
			resp,
			(const char *) "Standa8SMC3::read_Total_steps()", Tango::ERR);	
	
	if(resp.length() >= 3)
	{
		size_t pos = resp.find(":");
		string temp = resp.substr(pos +1,resp.length()-pos-3);   		//eliminate ':' and /d/a
		long val = strtol(temp.c_str(),0,16);
		*attr_Total_steps_read = val;
		DEBUG_STREAM << "Standa8SMC3::read_Total_steps(): read = "<< resp << " temp=" << temp << " val=" << val << endl;
		attr.set_value(attr_Total_steps_read);
	}
	else
		attr.set_quality(Tango::ATTR_INVALID);	
}*/

//+----------------------------------------------------------------------------
//
// method : 		Standa8SMC3::read_Steps_sx
// 
// description : 	Extract real attribute values for Steps_sx acquisition result.
//
//-----------------------------------------------------------------------------
/*void Standa8SMC3::read_Steps_sx(Tango::Attribute &attr)
{
	DEBUG_STREAM << "Standa8SMC3::read_Steps_sx(Tango::Attribute &attr) entering... "<< endl;
	string resp;
	abort_sleep((double(wr_sleep))/1000);
	if(WriteReadStanda(READ_STEPS_SX,"",resp, true))
		Tango::Except::throw_exception(
			(const char *) "Command failed",
			resp,
			(const char *) "Standa8SMC3::read_Steps_sx()", Tango::ERR);	
	
	if(resp.length() >= 3)
	{
		size_t pos = resp.find(":");
		string temp = resp.substr(pos +1,resp.length()-pos-3);		//eliminate ':' and /d/a
		long val = strtol(temp.c_str(),0,16);
		*attr_Steps_sx_read = val;
		DEBUG_STREAM << "Standa8SMC3::read_Steps_sx(): read = "<< resp << " temp=" << temp << " val=" << val << endl;
		attr.set_value(attr_Steps_sx_read);
	}
	else
		attr.set_quality(Tango::ATTR_INVALID);		
}*/

//+----------------------------------------------------------------------------
//
// method : 		Standa8SMC3::read_Steps_st
// 
// description : 	Extract real attribute values for Steps_st acquisition result.
//
//-----------------------------------------------------------------------------
/*void Standa8SMC3::read_Steps_st(Tango::Attribute &attr)
{
	DEBUG_STREAM << "Standa8SMC3::read_Steps_st(Tango::Attribute &attr) entering... "<< endl;
	string resp;
	abort_sleep((double(wr_sleep))/1000);
	if(WriteReadStanda(READ_STEPS_ST,"",resp, true))
		Tango::Except::throw_exception(
			(const char *) "Command failed",
			resp,
			(const char *) "Standa8SMC3::read_Steps_st()", Tango::ERR);	
	
	if(resp.length() >= 3)
	{
		size_t pos = resp.find(":");
		string temp = resp.substr(pos +1,resp.length()-pos-3);		//eliminate ':' and /d/a
		long val = strtol(temp.c_str(),0,16);
		*attr_Steps_st_read = val;
		DEBUG_STREAM << "Standa8SMC3::read_Steps_st(): read = "<< resp << " temp=" << temp << " val=" << val << endl;
		attr.set_value(attr_Steps_st_read);
	}
	else
		attr.set_quality(Tango::ATTR_INVALID);		
}*/

//+----------------------------------------------------------------------------
//
// method : 		Standa8SMC3::read_Stat
// 
// description : 	Extract real attribute values for Stat acquisition result.
//
//-----------------------------------------------------------------------------
void Standa8SMC3::read_Stat(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "Standa8SMC3::read_Stat(Tango::Attribute &attr) entering... "<< endl;
	*attr_Stat_read = stat;
	attr.set_value(attr_Stat_read);
}


//+----------------------------------------------------------------------------
//
// method : 		Standa8SMC3::read_Position
// 
// description : 	Extract real attribute values for Position acquisition result.
//
//-----------------------------------------------------------------------------
void Standa8SMC3::read_Position(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "Standa8SMC3::read_Position(Tango::Attribute &attr) entering... "<< endl;
	if(position_valid)
	{
		*attr_Position_read = (position_tmp + position_moving) / (double)positionRatio;
#if _MODULE_360
		do
		{
			if(*attr_Position_read >= 360)
				*attr_Position_read -= 360;
			else if(*attr_Position_read < 0)
				*attr_Position_read += 360;
		} while((*attr_Position_read >= 360) || (*attr_Position_read < 0));
#endif
		DEBUG_STREAM << "Standa8SMC3::read_Position() *attr_Position_read="<<*attr_Position_read<<" position_tmp= "<<position_tmp<<" position_moving=" << position_moving << " positionRatio="<<positionRatio<<endl;
		attr.set_value(attr_Position_read);
		attr.set_quality(Tango::ATTR_VALID);		
	}
	else
		attr.set_quality(Tango::ATTR_INVALID);
}

//+----------------------------------------------------------------------------
//
// method : 		Standa8SMC3::write_Position
// 
// description : 	Write Position attribute values to hardware.
//
//-----------------------------------------------------------------------------
void Standa8SMC3::write_Position(Tango::WAttribute &attr)
{
	DEBUG_STREAM << "Standa8SMC3::write_Position(Tango::WAttribute &attr) entering... "<< endl;
	attr.get_write_value(attr_Position_write);
	go_to_position(attr_Position_write);		
}

//+----------------------------------------------------------------------------
//
// method : 		Standa8SMC3::read_Acceleration
// 
// description : 	Extract real attribute values for Acceleration acquisition result.
//
//-----------------------------------------------------------------------------
void Standa8SMC3::read_Acceleration(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "Standa8SMC3::read_Acceleration(Tango::Attribute &attr) entering... "<< endl;
#ifdef _NOT_IN_THREAD 
	string resp;
	abort_sleep((double(wr_sleep))/1000);
	if(WriteReadStanda(READ_ACCELERATION,"",resp, true))
		Tango::Except::throw_exception(
			(const char *) "Command failed",
			resp,
			(const char *) "Standa8SMC3::read_Acceleration()", Tango::ERR);	
	
	if(resp.length() >= 5)
	{
		//size_t pos = resp.find(":12");//TODO: if 8smc1
		size_t pos = resp.find(":61");//TODO: if 8smc3
		string temp = resp.substr(pos +3,resp.length()-pos-5);		//eliminate ':12' and /d/a
		long val = strtol(temp.c_str(),0,16);
		*attr_Acceleration_read = val;
		//DEBUG_STREAM << "Standa8SMC3::read_Acceleration(): read = "<< resp << " temp=" << temp << " val=" << val << endl;
		attr.set_value(attr_Acceleration_read);
	}
	else
		attr.set_quality(Tango::ATTR_INVALID);
#else
	if(*attr_Acceleration_read != -1)
		attr.set_value(attr_Acceleration_read);
	else
		attr.set_quality(Tango::ATTR_INVALID);
#endif
}

//+----------------------------------------------------------------------------
//
// method : 		Standa8SMC3::write_Acceleration
// 
// description : 	Write Acceleration attribute values to hardware.
//
//-----------------------------------------------------------------------------
void Standa8SMC3::write_Acceleration(Tango::WAttribute &attr)
{
	DEBUG_STREAM << "Standa8SMC3::write_Acceleration(Tango::WAttribute &attr) entering... "<< endl;
	attr.get_write_value(attr_Acceleration_write);
	long lval = (long)attr_Acceleration_write;
	if(lval < 1 || lval > 255)
		Tango::Except::throw_exception(
			(const char *) "Command failed",
			(const char *) "Acceleration value have to be in range [1(max),255(min)]",
			(const char *) "Standa8SMC3::write_Acceleration()", Tango::ERR);	
	string resp;
	stringstream temp;
	string val;
	temp << hex << "0x" << attr_Acceleration_write;
	char temp_c[32];
	sprintf(temp_c,"%2.2X",lval);
	//val = strtol(temp.str().c_str(),0,16);
	temp >> val;
	if(WriteReadStanda(SET_ACCELERATION,temp_c,resp, true))
		Tango::Except::throw_exception(
			(const char *) "Command failed",
			resp,
			(const char *) "Standa8SMC3::write_Acceleration()", Tango::ERR);	
	DEBUG_STREAM << "Standa8SMC3::write_Acceleration(): write_resp = "<< resp << " attr_Acceleration_write=" << attr_Acceleration_write << " val=" << val << " temp=" << temp.str() << " temp_c=" << temp_c << endl;	
}

//+----------------------------------------------------------------------------
//
// method : 		Standa8SMC3::read_Speed
// 
// description : 	Extract real attribute values for Speed acquisition result.
//
//-----------------------------------------------------------------------------
void Standa8SMC3::read_Speed(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "Standa8SMC3::read_Speed(Tango::Attribute &attr) entering... "<< endl;
#ifdef _NOT_IN_THREAD 
	string resp;
	abort_sleep((double(wr_sleep))/1000);
	bool errmin, errmax;
	errmin=false;
	errmax=false;
	if(WriteReadStanda(READ_MIN_VELOCITY,"",resp, true))
		Tango::Except::throw_exception(
			(const char *) "Command failed",
			resp,
			(const char *) "Standa8SMC3::read_Speed()", Tango::ERR);	
	
	if(resp.length() >= 3)
	{
		size_t pos = resp.find(":");
		string temp = resp.substr(pos +1,resp.length()-pos-3);		//eliminate ':' and /d/a
		long val = strtol(temp.c_str(),0,16);
		if(val > 0 && val < 256)
		{
			attr_Speed_read[0] = speeds_table[val];
			errmin=false;
		}
		else
			errmin=true;
		DEBUG_STREAM << "Standa8SMC3::read_Speed(): MIN read = "<< resp << " temp=" << temp << " val=" << val << endl;
	}
	else
		errmin=true;
	if(WriteReadStanda(READ_MAX_VELOCITY,"",resp, true))
		Tango::Except::throw_exception(
			(const char *) "Command failed",
			resp,
			(const char *) "Standa8SMC3::read_Speed()", Tango::ERR);	
	
	if(resp.length() >= 3)
	{
		size_t pos = resp.find(":");
		string temp = resp.substr(pos +1,resp.length()-pos-3);		//eliminate ':' and /d/a
		long val = strtol(temp.c_str(),0,16);
		if(val > 0 && val < 256)
		{
			attr_Speed_read[1] = speeds_table[val];
			errmax=false;
		}
		else
			errmax=true;			
		DEBUG_STREAM << "Standa8SMC3::read_Speed(): MAX read = "<< resp << " temp=" << temp << " val=" << val << endl;

	}
	else
		errmax=true;		
	if(errmin || errmax)
		attr.set_quality(Tango::ATTR_INVALID);
	else
		attr.set_value(attr_Speed_read,2);
#else
	if((attr_Speed_read[0] != -1) && (attr_Speed_read[1] != -1))
		attr.set_value(attr_Speed_read,2);
	else
		attr.set_quality(Tango::ATTR_INVALID);
#endif	
}

//+----------------------------------------------------------------------------
//
// method : 		Standa8SMC3::write_Speed
// 
// description : 	Write Speed attribute values to hardware.
//
//-----------------------------------------------------------------------------
void Standa8SMC3::write_Speed(Tango::WAttribute &attr)
{
	DEBUG_STREAM << "Standa8SMC3::write_Speed(Tango::WAttribute &attr) entering... "<< endl;

	attr.get_write_value(attr_Speed_write);
	//double min_vel = 377.0 / (positionRatio * (double)step_factor);
	
	//if((long)(attr_Speed_write[1]* (positionRatio * (double)step_factor)) > maxAllowedSpeed || (long)(attr_Speed_write[1] * (positionRatio * (double)step_factor)) < 377)
	if((long)attr_Speed_write[1] > maxAllowedSpeed || (long)attr_Speed_write[1] < 377)
	{
		TangoSys_MemStream out_stream;
		//out_stream << "Max Velocity value have to be in range [" << min_vel << "," << ((double)maxAllowedSpeed)/(positionRatio * (double)step_factor) << "] degree/s" << ends;	
		out_stream << "Max Velocity value have to be in range [377," << maxAllowedSpeed << "] Tiks Hz" << ends;
		Tango::Except::throw_exception(
			(const char *) "Command failed",
			out_stream.str(),
			(const char *) "Standa8SMC3::write_Speed()", Tango::ERR);
	}	
	//if((long)attr_Speed_write[0] > (long)attr_Speed_write[1] || (long)(attr_Speed_write[0] * (positionRatio * (double)step_factor)) < 377)
	if((long)attr_Speed_write[0] > (long)attr_Speed_write[1] || (long)attr_Speed_write[0] < 377)
	{
		TangoSys_MemStream out_stream;
		out_stream << "Min Velocity value have to be in range [377," << (long)attr_Speed_write[1] << "] Tiks Hz" << ends;
		Tango::Except::throw_exception(
			(const char *) "Command failed",
			out_stream.str(),
			(const char *) "Standa8SMC3::write_Speed()", Tango::ERR);
	}
	long lval = 1;
	for(int i=1;i<256;i++)
	{
		//if((long)(attr_Speed_write[0] * (positionRatio * (double)step_factor)) >= speeds_table[lval])
		if(attr_Speed_write[0] >= speeds_table[lval])
			break;
		lval = i;
		//DEBUG_STREAM << "Standa8SMC3::write_Speed(): MIN -	settinga lval to " << lval << endl;	
	}	
	string resp;
	//stringstream temp;
	//string val;
	//temp << hex << "0x" << attr_Acceleration_write[0];
	char temp_c[32];
	sprintf(temp_c,"%2.2X",lval);
	//val = strtol(temp.str().c_str(),0,16);
	//temp >> val;
	if(WriteReadStanda(SET_MIN_VELOCITY,temp_c,resp, true))
		Tango::Except::throw_exception(
			(const char *) "Command failed",
			resp,
			(const char *) "Standa8SMC3::write_Speed()", Tango::ERR);	
	DEBUG_STREAM << "Standa8SMC3::write_Speed(): MIN - write_resp = "<< resp << " write_Speed=" << attr_Speed_write[0] << " temp_c=" << temp_c << endl;

	abort_sleep((double(wr_sleep))/1000);
	lval = 1;
	for(int i=1;i<256;i++)
	{
		//if((long)(attr_Speed_write[1] * (positionRatio * (double)step_factor)) >= speeds_table[lval])
		if(attr_Speed_write[1] >= speeds_table[lval])
			break;
		lval = i;
		//DEBUG_STREAM << "Standa8SMC3::write_Speed(): MAX -	setting lval to " << lval << endl;		
	}
	sprintf(temp_c,"%2.2X",lval);
	if(WriteReadStanda(SET_MAX_VELOCITY,temp_c,resp, true))
		Tango::Except::throw_exception(
			(const char *) "Command failed",
			resp,
			(const char *) "Standa8SMC3::write_Speed()", Tango::ERR);	
	DEBUG_STREAM << "Standa8SMC3::write_Speed(): MAX - write_resp = "<< resp << " write_Speed=" << attr_Speed_write[1] << " temp_c=" << temp_c << endl;	
	
}

//+----------------------------------------------------------------------------
//
// method : 		Standa8SMC3::read_Accuracy
// 
// description : 	Extract real attribute values for Accuracy acquisition result.
//
//-----------------------------------------------------------------------------
void Standa8SMC3::read_Accuracy(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "Standa8SMC3::read_Accuracy(Tango::Attribute &attr) entering... "<< endl;
#ifdef _NOT_IN_THREAD 
	string resp;
	abort_sleep((double(wr_sleep))/1000);
	if(WriteReadStanda(READ_MICRO_STEP_CONF,"",resp, true))
		Tango::Except::throw_exception(
			(const char *) "Command failed",
			resp,
			(const char *) "Standa8SMC3::read_Acceleration()", Tango::ERR);	
	
	if(resp.length() >= 4)
	{
		size_t pos = resp.find(":0");
		string temp = resp.substr(pos +2,resp.length()-pos-4);		//eliminate ':0' and /d/a
		long val = strtol(temp.c_str(),0,16);
		if(val==1)
			step_factor = 1;
		else if(val==2)
			step_factor = 2;
		else if(val==3)
			step_factor = 4;
		else if(val==4)
			step_factor = 8;			
		*attr_Accuracy_read = val;
		//DEBUG_STREAM << "Standa8SMC3::read_Accuracy(): read = "<< resp << " temp=" << temp << " val=" << val << endl;
		attr.set_value(attr_Accuracy_read);
	}
	else
		attr.set_quality(Tango::ATTR_INVALID);
#else
	if(*attr_Accuracy_read != -1)
		attr.set_value(attr_Accuracy_read);
	else
		attr.set_quality(Tango::ATTR_INVALID);
#endif
}

//+----------------------------------------------------------------------------
//
// method : 		Standa8SMC3::write_Accuracy
// 
// description : 	Write Accuracy attribute values to hardware.
//
//-----------------------------------------------------------------------------
void Standa8SMC3::write_Accuracy(Tango::WAttribute &attr)
{
	DEBUG_STREAM << "Standa8SMC3::write_Accuracy(Tango::WAttribute &attr) entering... "<< endl;
	unsigned int val;
	string resp;
	attr.get_write_value(attr_Accuracy_write);
	val = (unsigned int)attr_Accuracy_write;
	if(val != 1 && val != 2 && val != 3 && val != 4)
		Tango::Except::throw_exception(
			(const char *) "Command failed",
			(const char *) "Accuracy have to be 1(full step), 2(half step), 3(1/4 step) or 4(1/8 step)",
			(const char *) "Standa8SMC3::write_Accuracy()", Tango::ERR);	
	if(val==1)
		step_factor = 1;
	else if(val==2)
		step_factor = 2;
	else if(val==3)
		step_factor = 4;
	else if(val==4)
		step_factor = 8;	
	//TODO: control input value
	char temp_c[32];
	sprintf(temp_c,"%d",val);
	//val = strtol(temp.str().c_str(),0,16);
	//temp >> val;
	if(WriteReadStanda(SET_MICRO_STEP_CONF,temp_c,resp, true))
		Tango::Except::throw_exception(
			(const char *) "Command failed",
			resp,
			(const char *) "Standa8SMC3::write_Accuracy()", Tango::ERR);	
	DEBUG_STREAM << "Standa8SMC3::write_Accuracy(): write_resp = "<< resp << " write_Accuracy=" << attr_Accuracy_write << " temp_c=" << temp_c << endl;		
}



//+------------------------------------------------------------------
/**
 *	method:	Standa8SMC3::forward
 *
 *	description:	method to execute "Forward"
 *
 *
 */
//+------------------------------------------------------------------
//void Standa8SMC3::forward()
//{
//	DEBUG_STREAM << "Standa8SMC3::forward(): entering... !" << endl;
//
//	//	Add your own code to control device here
//	string resp;
//	if(get_state() != Tango::ON)
//		Tango::Except::throw_exception(
//			(const char *) "Command failed",
//			(const char *) "Motor have to be in ON state to move",
//			(const char *) "Standa8SMC3::forward()", Tango::ERR);	
//	abort_sleep((double(wr_sleep))/1000);	
//	if(WriteReadStanda(SET_ROTATION_WAY,"1",resp, true))
//		Tango::Except::throw_exception(
//			(const char *) "Command failed",
//			resp,
//			(const char *) "Standa8SMC3::forward()", Tango::ERR);	
//	DEBUG_STREAM << "Standa8SMC3::forward(): write = " << SET_ROTATION_WAY << " read = "<< resp << endl;
//	abort_sleep((double(wr_sleep))/1000);
//	if(WriteReadStanda(SET_STEP_NUMBER,"004100",resp, true))
//		Tango::Except::throw_exception(
//			(const char *) "Command failed",
//			resp,
//			(const char *) "Standa8SMC3::forward()", Tango::ERR);	
//	DEBUG_STREAM << "Standa8SMC3::forward(): write = " << SET_STEP_NUMBER << " read = "<< resp << endl;
//	readloop->mutex_sleep->lock();
//	readloop->pausesec_dfl = ((int)th_sleep)/1000;					//poll status fast when not moving
//	readloop->pausenano_dfl = ((th_sleep)-readloop->pausesec_dfl*1000)*1000000;
//	readloop->mutex_sleep->unlock();
//	abort_sleep((double(wr_sleep))/1000);
//	if(WriteReadStanda(CMD_MOVE,"",resp, true))
//		Tango::Except::throw_exception(
//			(const char *) "Command failed",
//			resp,
//			(const char *) "Standa8SMC3::forward()", Tango::ERR);	
//	DEBUG_STREAM << "Standa8SMC3::forward(): write = " << CMD_MOVE << " read = "<< resp << endl;			
//
//}

//+------------------------------------------------------------------
/**
 *	method:	Standa8SMC3::backward
 *
 *	description:	method to execute "Backward"
 *
 *
 */
//+------------------------------------------------------------------
//void Standa8SMC3::backward()
//{
//	DEBUG_STREAM << "Standa8SMC3::backward(): entering... !" << endl;
//
//	//	Add your own code to control device here
//	string resp;
//	if(get_state() != Tango::ON)
//		Tango::Except::throw_exception(
//			(const char *) "Command failed",
//			(const char *) "Motor have to be in ON state to move",
//			(const char *) "Standa8SMC3::backward()", Tango::ERR);	
//	abort_sleep((double(wr_sleep))/1000);	
//	if(WriteReadStanda(SET_ROTATION_WAY,"0",resp, true))
//		Tango::Except::throw_exception(
//			(const char *) "Command failed",
//			resp,
//			(const char *) "Standa8SMC3::backward()", Tango::ERR);	
//	DEBUG_STREAM << "Standa8SMC3::backward(): write = " << SET_ROTATION_WAY << " read = "<< resp << endl;
//	abort_sleep((double(wr_sleep))/1000);
//	if(WriteReadStanda(SET_STEP_NUMBER,"004000",resp, true))
//		Tango::Except::throw_exception(
//			(const char *) "Command failed",
//			resp,
//			(const char *) "Standa8SMC3::backward()", Tango::ERR);	
//	DEBUG_STREAM << "Standa8SMC3::backward(): write = " << SET_STEP_NUMBER << " read = "<< resp << endl;
//	readloop->mutex_sleep->lock();
//	readloop->pausesec_dfl = ((int)th_sleep)/1000;					//poll status fast when not moving
//	readloop->pausenano_dfl = ((th_sleep)-readloop->pausesec_dfl*1000)*1000000;
//	readloop->mutex_sleep->unlock();	
//	abort_sleep((double(wr_sleep))/1000);
//	if(WriteReadStanda(CMD_MOVE,"",resp, true))
//		Tango::Except::throw_exception(
//			(const char *) "Command failed",
//			resp,
//			(const char *) "Standa8SMC3::backward()", Tango::ERR);	
//	DEBUG_STREAM << "Standa8SMC3::backward(): write = " << CMD_MOVE << " read = "<< resp << endl;
//}

//+------------------------------------------------------------------
/**
 *	method:	Standa8SMC3::stop
 *
 *	description:	method to execute "Stop"
 *
 *
 */
//+------------------------------------------------------------------
void Standa8SMC3::stop()
{
	DEBUG_STREAM << "Standa8SMC3::stop(): entering... !" << endl;

	//	Add your own code to control device here
	string resp;
	abort_sleep((double(wr_sleep))/1000);	
	if(WriteReadStanda(CMD_STOP,"",resp, true))
		Tango::Except::throw_exception(
			(const char *) "Command failed",
			resp,
			(const char *) "Standa8SMC3::stop()", Tango::ERR);	
	//position_tmp = 0;
	//position_valid = false;			
	DEBUG_STREAM << "Standa8SMC3::stop(): write = " << CMD_STOP << " read = "<< resp << endl;
}

//+------------------------------------------------------------------
/**
 *	method:	Standa8SMC3::motor_off
 *
 *	description:	method to execute "MotorOFF"
 *	disable the power amplifier of the motor
 *
 *
 */
//+------------------------------------------------------------------
void Standa8SMC3::motor_off()
{
	DEBUG_STREAM << "Standa8SMC3::motor_off(): entering... !" << endl;

	//	Add your own code to control device here
	string resp;
	abort_sleep((double(wr_sleep))/1000);	
	if(WriteReadStanda(CMD_OFF,"",resp, true))
		Tango::Except::throw_exception(
			(const char *) "Command failed",
			resp,
			(const char *) "Standa8SMC3::motor_off()", Tango::ERR);	
	DEBUG_STREAM << "Standa8SMC3::motor_off(): write = " << CMD_OFF << " read = "<< resp << endl;
	if(get_state()!=Tango::OFF)
	{
		set_state(Tango::OFF);	
		set_status("Motor OFF");
		/*vector<string> filterable_names;
		vector<double> filterable_values;
		push_event("State", filterable_names, filterable_values, &device_state);*/
	}		
	old_state = device_state;
	old_status = device_status;
}

//+------------------------------------------------------------------
/**
 *	method:	Standa8SMC3::motor_on
 *
 *	description:	method to execute "MotorON"
 *	enables the power amplifier of the motor
 *
 *
 */
//+------------------------------------------------------------------
void Standa8SMC3::motor_on()
{
	DEBUG_STREAM << "Standa8SMC3::motor_on(): entering... !" << endl;

	//	Add your own code to control device here
	string resp;
	abort_sleep((double(wr_sleep))/1000);	
	if(WriteReadStanda(CMD_ON,"",resp, true))
		Tango::Except::throw_exception(
			(const char *) "Command failed",
			resp,
			(const char *) "Standa8SMC3::motor_on()", Tango::ERR);	
	DEBUG_STREAM << "Standa8SMC3::motor_on(): write = " << CMD_ON << " read = "<< resp << endl;
	abort_sleep((double(wr_sleep))/1000);
	char lsp[10];
	sprintf(lsp, "%d",limit_switch_polarity);
	if(WriteReadStanda(SET_LS_POLARITY,lsp,resp, true))
		Tango::Except::throw_exception(
			(const char *) "Command failed",
			resp,
			(const char *) "Standa8SMC3::motor_on()", Tango::ERR);	
	DEBUG_STREAM << "Standa8SMC3::motor_on(): write = " << SET_LS_POLARITY << limit_switch_polarity<<" read = "<< resp << endl;
	if(get_state()!=Tango::ON)
	{
		set_state(Tango::ON);	
		set_status("Motor ON");
		/*vector<string> filterable_names;
		vector<double> filterable_values;
		push_event("State", filterable_names, filterable_values, &device_state);*/
	}		
	old_state = device_state;
	old_status = device_status;
}

//+------------------------------------------------------------------
/**
 *	method:	Standa8SMC3::motor_standby
 *
 *	description:	method to execute "MotorSTANDBY"
 *
 *
 */
//+------------------------------------------------------------------
void Standa8SMC3::motor_standby()
{
	DEBUG_STREAM << "Standa8SMC3::motor_standby(): entering... !" << endl;

	//	Add your own code to control device here
	string resp;
	abort_sleep((double(wr_sleep))/1000);	
	if(WriteReadStanda(CMD_STANDBY,"",resp, true))
		Tango::Except::throw_exception(
			(const char *) "Command failed",
			resp,
			(const char *) "Standa8SMC3::motor_standby()", Tango::ERR);	
	DEBUG_STREAM << "Standa8SMC3::motor_standby(): write = " << CMD_STANDBY << " read = "<< resp << endl;
	if(get_state()!=Tango::STANDBY)
	{
		set_state(Tango::STANDBY);	
		set_status("Motor STANDBY");
		/*vector<string> filterable_names;
		vector<double> filterable_values;
		push_event("State", filterable_names, filterable_values, &device_state);*/
	}		
	old_state = device_state;
	old_status = device_status;		
}

//+------------------------------------------------------------------
/**
 *	method:	Standa8SMC3::define_position
 *
 *	description:	method to execute "DefinePosition"
 *
 * @param	argin	
 *
 */
//+------------------------------------------------------------------
/*void Standa8SMC3::define_position(Tango::DevDouble argin)
{
	DEBUG_STREAM << "Standa8SMC3::define_position(): entering... !" << endl;

	//	Add your own code to control device here

}*/


//+------------------------------------------------------------------
//
//	method:			Standa8SMC3::WriteReadStanda
//
//	description:	
//
//-------------------------------------------------------------------
int Standa8SMC3::WriteReadStanda(string cmd, string value, string &argout, bool has_resp)
{
	//DEBUG_STREAM << "Standa8SMC3::WriteReadStanda(): entering... !" << endl;
	if(abortflag)
		return 0;
	mutex->lock();
	char outbuf[256],
	inbuf[256],respbuf[256];
	stringstream temp;
	bool ok = false;
	int nretries = 0;
	argout = string("");
	
	sprintf(outbuf,"%s%d%s%s\r",PREFIX, axisNumber, cmd.c_str(), value.c_str());	
	int count=0;
#if 0	//TEST
	try {
		while(devserial->is_locked() && ++count<100 && !abortflag)
		{
			usleep(10000);
		}

		if(devserial->is_locked())
		{
			TangoSys_MemStream out_stream;
			out_stream << "TIMEOUT locking serial cmd = " << cmd << ends;
			WARN_STREAM << "Standa8SMC3::WriteReadStanda(): " << out_stream.str() << endl;
			argout = out_stream.str();
			mutex->unlock();
			return -1;
		}
	}
	catch(Tango::DevFailed &e) {
		TangoSys_MemStream out_stream;
		out_stream << "ERROR LOCKING serial cmd = " << cmd << ", err = " << e.errors[0].desc << ends;
		WARN_STREAM << "Standa8SMC3::WriteReadStanda(): " << out_stream.str() << endl;
		argout = out_stream.str();
		mutex->unlock();
		return -1;
	}
#endif
	DEBUG_STREAM << "Standa8SMC3::WriteReadStanda(): BEFORE LOCKING DEVSERIAL" << endl;
#if 1
	do
	{
		try {
			devserial->lock();
			DEBUG_STREAM << "Standa8SMC3::WriteReadStanda(): OK LOCKING DEVSERIAL" << endl;
		}
		catch(Tango::DevFailed &e) {
			//WARN_STREAM << "Standa8SMC3::WriteReadStanda(): ERROR LOCKING DEVSERIAL: " << e.errors[0].desc << endl;
			usleep(9000);
		}
	}
	while(!devserial->is_locked_by_me() && ++count<210 && !abortflag);
	if(!devserial->is_locked_by_me() || abortflag)
	{
		TangoSys_MemStream out_stream;
		out_stream << "TIMEOUT locking serial cmd = " << cmd << " status="<< devserial->locking_status()<< endl;;
		WARN_STREAM << "Standa8SMC3::WriteReadStanda(): " << out_stream.str() << endl;
		argout = out_stream.str();
		mutex->unlock();
		return -1;
	}
#else
	try {
		devserial->lock();
		DEBUG_STREAM << "Standa8SMC3::WriteReadStanda(): OK LOCKING DEVSERIAL" << endl;
	}
	catch(Tango::DevFailed &e) {
		WARN_STREAM << "Standa8SMC3::WriteReadStanda(): ERROR LOCKING DEVSERIAL: " << e.errors[0].desc << endl;
	}
#endif
	//DEBUG_STREAM << "Standa8SMC3::WriteReadStanda(): AFTER LOCKING DEVSERIAL status="<< devserial->locking_status()<< endl;
	try
	{
		if(!socket)
			write_serial(outbuf,strlen(outbuf));
		else
			write_socket2(outbuf,strlen(outbuf));
		abort_sleep((double(wr_sleep))/1000);  //abort_sleep(sec)	
		//read_serial(inbuf);			
	}
	catch(Tango::DevFailed &e) {
		TangoSys_MemStream out_stream;
		out_stream << "Error writing cmd = " << cmd << ", err = " << e.errors[0].desc << ends;
		WARN_STREAM << "Standa8SMC3::WriteReadStanda(): " << out_stream.str() << endl;
		argout = out_stream.str();
		try
		{
			devserial->unlock();
			DEBUG_STREAM << "Standa8SMC3::WriteReadStanda(): AFTER UNLOCKING DEVSERIAL" << endl;
		}
		catch(Tango::DevFailed &e2)
		{
			WARN_STREAM << "Standa8SMC3::WriteReadStanda(): ERROR UNLOCKING DEVSERIAL: " << e2.errors[0].desc << endl;
		}
		mutex->unlock();
		return -1;							
	}
	if(!has_resp)
	{
		try
		{
			devserial->unlock();
			DEBUG_STREAM << "Standa8SMC3::WriteReadStanda(): AFTER UNLOCKING DEVSERIAL" << endl;
		}
		catch(Tango::DevFailed &e2)
		{
			WARN_STREAM << "Standa8SMC3::WriteReadStanda(): ERROR UNLOCKING DEVSERIAL: " << e2.errors[0].desc << endl;
		}
		mutex->unlock();
		return 0;
	}
	memset(inbuf,0,256);	
	try {	
		if(!socket)
			read_serial(inbuf);
		else
			read_socket2(inbuf);			
	}
	catch(Tango::DevFailed &e) {
		TangoSys_MemStream out_stream;
		out_stream << "Error cmd '" << outbuf << "' failed!!, err reading response= " << e.errors[0].desc << ends;	
		WARN_STREAM << "Standa8SMC3::WriteReadStanda(): " << out_stream.str() << endl;
		argout = out_stream.str();
		try
		{
			devserial->unlock();
			DEBUG_STREAM << "Standa8SMC3::WriteReadStanda(): AFTER UNLOCKING DEVSERIAL" << endl;
		}
		catch(Tango::DevFailed &e2)
		{
			WARN_STREAM << "Standa8SMC3::WriteReadStanda(): ERROR UNLOCKING DEVSERIAL: " << e2.errors[0].desc << endl;
		}
		mutex->unlock();
		return -1;		
	}
	if((cmd == CMD_MOVE) || (cmd == CMD_OFF) || (cmd == CMD_ON) || (cmd == CMD_STANDBY) || (cmd == CMD_STOP) ||
		(cmd == SET_MIN_VELOCITY) || (cmd == SET_MAX_VELOCITY) || (cmd == SET_ACCELERATION) || (cmd == SET_MICRO_STEP_CONF) ||
		(cmd == SET_LIMIT_ON) || (cmd == SET_LIMIT_OFF) || (cmd == SET_ROTATION_WAY) || (cmd == SET_STEP_NUMBER) || (cmd == SET_OFFSET_STEP) ||
		(cmd == ENABLE_SWITCH) || (cmd == CMD_RESET) || (cmd == SET_LS_POLARITY))
		sprintf(respbuf,"%d:%s", axisNumber, cmd.c_str());
	else if((cmd == READ_MIN_VELOCITY) || (cmd == READ_MAX_VELOCITY) || (cmd == READ_MICRO_STEP_CONF) || (cmd == READ_STEPS) ||
		(cmd == READ_STEPS_SX) || (cmd == READ_STEPS_ST) || (cmd == READ_STATUS) || (cmd == READ_LIMIT_1) || (cmd == READ_LIMIT_2))
		sprintf(respbuf,"%d:", axisNumber);
	else if((cmd == READ_ACCELERATION))
		//sprintf(respbuf,"%d:12", axisNumber);	//TODO: if 8smc1
		sprintf(respbuf,"%d:61", axisNumber);	//TODO: if 8smc3
	if(strncmp(inbuf,respbuf,strlen(respbuf)) != 0)
	{
		TangoSys_MemStream out_stream;
		out_stream << "Error cmd '" << outbuf << "' expecting answer '" << respbuf << "', received= " << inbuf << ends;	
		WARN_STREAM << "Standa8SMC3::WriteReadStanda(): " << out_stream.str() << endl;
		argout = out_stream.str();
		if(!socket)
		{
			Tango::DeviceData Din;
			Tango::DevLong flush = 2;  //flush input and output
			Din << flush;
			try	{
				devserial->command_inout("DevSerFlush",Din);
			}
			catch(Tango::DevFailed &e3) {
				WARN_STREAM << "Standa8SMC3::WriteReadStanda(): ERROR FLUSHING DEVSERIAL: " << e3.errors[0].desc << endl;
			}
		}
		else
		{
			Tango::DeviceData Dout;
			try {
				Dout = devserial->command_inout("Read");
				if(!Dout.is_empty())
				{
					string out;
					Dout >> out;
					DEBUG_STREAM << "Standa8SMC3::WriteReadStanda(): initially read from socket: " <<  out << endl;
				}
			}
			catch(Tango::DevFailed &e3) {
				TangoSys_MemStream out_stream;
				out_stream << "Read command from SocketDeviceServer failed";
				DEBUG_STREAM << "Standa8SMC3::WriteReadStanda(): " <<  out_stream.str() << endl;
			}
		}
		try
		{
			devserial->unlock();
			DEBUG_STREAM << "Standa8SMC3::WriteReadStanda(): AFTER UNLOCKING DEVSERIAL" << endl;
		}
		catch(Tango::DevFailed &e2)
		{
			WARN_STREAM << "Standa8SMC3::WriteReadStanda(): ERROR UNLOCKING DEVSERIAL: " << e2.errors[0].desc << endl;
		}
		mutex->unlock();
		return -1;
	}
/*	if(inbuf[0] != ACK)
	{
		TangoSys_MemStream out_stream;
		out_stream << "Error in cmd = " << cmd << "; not Ack response ";
		stringstream temp;
		for (int i = 0; i<strlen(inbuf); i++)
			temp << showbase << hex << (int)inbuf[i] << " ";
		//temp << ends;
		WARN_STREAM << "Im540::WriteReadIm540(): " << out_stream.str() << "= " << temp.str() << endl;
		rdata.msg = out_stream.str();
		if(cmd != ERR_CMD)
		{
			WriteReadIm540(ERR_CMD,err_data);
			if(err_data.msg != "")
			{
				rdata.msg += " -> " + err_data.msg;
			}
		}
		return -1;		
	}*/
/*	if(cmd == SENSOR_CONTROL_ACTIVATE)		//command without response
	{
		rdata.msg= "";
		return 0;
	}*/
	if(get_state() == Tango::FAULT)
	{
		set_state(old_state);
		set_status(old_status);
	}
	argout = string(inbuf);
	try
	{
		devserial->unlock();
		DEBUG_STREAM << "Standa8SMC3::WriteReadStanda(): AFTER UNLOCKING DEVSERIAL" << endl;
	}
	catch(Tango::DevFailed &e2)
	{
		WARN_STREAM << "Standa8SMC3::WriteReadStanda(): ERROR UNLOCKING DEVSERIAL: " << e2.errors[0].desc << endl;
	}
	mutex->unlock();
	return 0;
}

//+------------------------------------------------------------------
//
//	method:			Standa8SMC3::write_serial
//
//	description:	method to execute "write_serial"
//					Write a string to serial interface
//
//-------------------------------------------------------------------
void Standa8SMC3::write_serial(char *write_str, long size)
//void Chiller::write_serial(string write_str)
{
	//DEBUG_STREAM << "Im540::write_serial(): entering... !" << endl;

	//	Add your own code to control device here
	Tango::DeviceData Din,Din2;
	int i;
	Tango::DevLong flush = 2;
	Tango::DevVarCharArray *input = new Tango::DevVarCharArray();
	input->length(size);
//	input->length(write_str.length());
	stringstream temp;
	
//	for (int i=0; i < write_str.length(); i++)
	for (int i=0; i < size; i++)
	{
		//(*input)[i] = (unsigned char) write_str.at(i);
		(*input)[i] = write_str[i];
		temp << showbase << hex << (int)(*input)[i] << " ";
	}

	//DEBUG_STREAM << "Standa8SMC3::write_serial(): writing=" << temp.str() << endl;

	try {
		Din << input;
		devserial->command_inout("DevSerWriteChar",Din);
/*		Din2 << flush;
		devserial->command_inout("DevSerFlush",Din2);*/
	}
	catch(Tango::DevFailed e) {	
			TangoSys_MemStream out_stream;
			out_stream << "Write command to SerialDeviceServer failed" << ends;
			//set_state(Tango::FAULT);
			//set_status("Errors from serialdevice " + serialdevice);		
			Tango::Except::re_throw_exception(e,
				(const char *) "Error writing serial",
				out_stream.str(),
				(const char *) "Standa8SMC3::write_serial()", Tango::ERR);
	}
	
//	set_state(Tango::ON);
//	set_status("Connected to serialdevice " + serialdevice);
}

//+------------------------------------------------------------------
//
//	method:			Standa8SMC3::read_serial
//
//	description:	method to execute "read_serial"
//					Read from serial interface 
//
//-------------------------------------------------------------------
void Standa8SMC3::read_serial(char *read_str/*, long read_len*/)
{
	//DEBUG_STREAM << "Im540::read_serial(): entering... !" << endl;

	//	Add your own code to control device here
	Tango::DevVarLongStringArray strin;
	string input, strout, strout_filt;
	Tango::DeviceData Din,Dout;
	const Tango::DevVarCharArray *out;
	//long attr_read_param = SL_RAW;
	long attr_read_param;
	//attr_read_param = SAALEN << 8;
	//attr_read_param = 1 << 8;   //provo a leggere solo un carattere
	//attr_read_param |= SL_NCHAR;
	attr_read_param = SL_LINE;
	
	//attr_read_param = 2 << 8;   //provo a leggere per 2 tentativi
	//attr_read_param |= SL_RETRY;
	long temp_len;
	int i;

	try {	
		Din << attr_read_param;
		Dout = devserial->command_inout("DevSerReadChar",Din);
	}
	catch(Tango::DevFailed e) {
		TangoSys_MemStream out_stream;
		out_stream << "DevSerReadNChar command to SerialDeviceServer failed" << ends;
		set_state(Tango::FAULT);
		set_status("Errors from serialdevice " + serialdevice);
		WARN_STREAM << "Standa8SMC3::read_serial(): " <<  out_stream.str() << endl;			
		Tango::Except::re_throw_exception(e,
			(const char *) "Error reading serial",
			out_stream.str(),
			(const char *) "Standa8SMC3::read_serial()", Tango::ERR);
	}
	if(!Dout.is_empty())
		Dout >> out;
	else
	{
		//set_state(Tango::FAULT);
		//set_status("Errors from serialdevice " + serialdevice);			
		Tango::Except::throw_exception(
			(const char *) "Serialline timeout",
			(const char *) "No data from serialline",
			(const char *) "Standa8SMC3::read_serial()", Tango::ERR);		
	}

	memset(read_str,0,256);
	temp_len = out->length();
	if(temp_len > 256)	
	{
		WARN_STREAM << "Standa8SMC3::read_serial(): read buffer overflow, " << 
			temp_len << " > " <<  256 << endl;
		temp_len = 256;
	}
	stringstream temp;
	for (int i = 0; i < temp_len; i++)
	{
		read_str[i] = (*out)[i];
		temp << showbase << hex << (int)(*out)[i] << " ";
	}
	//DEBUG_STREAM << "Standa8SMC3::read_serial(): read=" << temp.str() << endl;
}

//+------------------------------------------------------------------
//
//	method:			Standa8SMC3::write_socket
//
//	description:	method to execute "write_socket"
//					Write a string to socket interface
//
//-------------------------------------------------------------------
void Standa8SMC3::write_socket(char *write_str, long size)
{
	DEBUG_STREAM << "Standa8SMC3::write_socket(): entering... =" << write_str << endl;

	//	Add your own code to control device here
	Tango::DeviceData Din,Din2;
	int i;
	stringstream temp;
	string cmd(write_str);

	for (int i=0; i < size; i++)
	{
		temp << showbase << hex << (int)write_str[i] << " ";
	}

	DEBUG_STREAM << "Standa8SMC3::write_socket(): writing=" << temp.str() << endl;

	try {
		Din << cmd;
		devserial->command_inout("Write",Din);
	}
	catch(Tango::DevFailed e) {
			TangoSys_MemStream out_stream;
			out_stream << "Write command to SocketDeviceServer failed" << ends;
			set_state(Tango::FAULT);
			Tango::DevState st = Tango::FAULT;
			push_change_event("state", &st);
			set_status("Errors from socketdevice " + serialdevice);
			Tango::Except::re_throw_exception(e,
				(const char *) "Error writing socket",
				out_stream.str(),
				(const char *) "Standa8SMC3::write_socket()", Tango::ERR);
	}
}

void Standa8SMC3::write_socket2(char *write_str, long size)
{
	char cmd[64];
	memset(cmd,0,64);
	sprintf(cmd, "%s",write_str);

	try
	{

		{

			Tango::DevVarCharArray dvca;
			dvca.length(strlen(cmd)+1);
			for (unsigned int i=0; i<strlen(cmd); ++i)
				dvca[i] = cmd[i];
			dvca[strlen(cmd)] = '\r';

			Tango::DeviceData din;
			din << dvca;
			devserial->command_inout("Write", din);
		}

		DEBUG_STREAM << __func__<<": sent command: " << cmd;
	}
	catch(Tango::DevFailed &e)
	{
			TangoSys_MemStream out_stream;
			out_stream << "Write command to SocketDeviceServer failed" << ends;
			set_state(Tango::FAULT);
			Tango::DevState st = Tango::FAULT;
			push_change_event("state", &st);
			set_status("Errors from socketdevice " + serialdevice);
			Tango::Except::re_throw_exception(e,
				(const char *) "Error writing socket2",
				out_stream.str(),
				(const char *) "Standa8SMC3::write_socket2()", Tango::ERR);
	}
	catch(...)
	{
		INFO_STREAM << __func__<<": NON Tango Exception sending command";
	}
}

//+------------------------------------------------------------------
//
//	method:			Standa8SMC3::read_socket
//
//	description:	method to execute "read_socket"
//					Read from socket interface
//
//-------------------------------------------------------------------
void Standa8SMC3::read_socket(char *read_str/*, long read_len*/)
{
	//DEBUG_STREAM << "Standa8SMC3::read_socket(): entering... !" << endl;

	//	Add your own code to control device here
	Tango::DevVarLongStringArray strin;
	string input, strout, strout_filt;
	Tango::DeviceData Din,Dout;
	string out;

	long temp_len;
	int i;

	try {
		Dout = devserial->command_inout("Read");
	}
	catch(Tango::DevFailed e) {
		TangoSys_MemStream out_stream;
		out_stream << "DevSerReadNChar command to SocketDeviceServer failed";
		set_state(Tango::FAULT);
		set_status("Errors from socketdevice " + serialdevice);
		WARN_STREAM << "MenloLaser::read_socket(): " <<  out_stream.str() << endl;
		Tango::Except::re_throw_exception(e,
			(const char *) "Error reading socket",
			out_stream.str(),
			(const char *) "Standa8SMC3::read_socket()", Tango::ERR);
	}
	if(!Dout.is_empty())
		Dout >> out;
	else
	{
		set_state(Tango::FAULT);
		set_status("Errors from scoketdevice " + serialdevice);
		Tango::Except::throw_exception(
			(const char *) "Serialline timeout",
			(const char *) "No data from socket",
			(const char *) "Standa8SMC3::read_socket()", Tango::ERR);
	}

	memset(read_str,0,256);
	temp_len = out.length();
	if(temp_len > 256)
	{
		WARN_STREAM << "Standa8SMC3::read_socket(): read buffer overflow, " <<
			temp_len << " > " <<  256 << endl;
		temp_len = 256;
	}
	stringstream temp;
	for (int i = 0; i < temp_len; i++)
	{
		read_str[i] = out[i];
		temp << showbase << hex << (int)out[i] << " ";
	}
	DEBUG_STREAM << "Standa8SMC3::read_socket(): read=" << temp.str() << endl;
}

void Standa8SMC3::read_socket2(char *read_str/*, long read_len*/)
{
	string resp("");
	bool read_err=false;
	try
	{
		{
			Tango::DeviceData din2, dout;

			{
				Tango::DevVarCharArray *delim =  new Tango::DevVarCharArray();
				delim->length(1);
				(*delim)[0] = '\r';//OK
				din2 << delim;
			}
			usleep(50000);	//80ms

			{
				const Tango::DevVarCharArray *dtr;
				dout = devserial->command_inout("ReadUntil", din2);
				dout >> dtr;

				for (unsigned int i=0; i<dtr->length(); ++i)
					resp += (*dtr)[i];
			}
			DEBUG_STREAM << __func__<<": read response='"<<resp << "'";
			DEBUG_STREAM << __func__<<": read response len="<<resp.length();

		}
	}
	catch(Tango::DevFailed &e)
	{
		TangoSys_MemStream out_stream;
		out_stream << "Socket2::ReadUntil command failed";
		set_state(Tango::FAULT);
		set_status("Errors from socketdevice " + serialdevice);
		WARN_STREAM << "MenloLaser::read_socket(): " <<  out_stream.str() << endl;
		Tango::Except::re_throw_exception(e,
			(const char *) "Error reading socket",
			out_stream.str(),
			(const char *) "Standa8SMC3::read_socket2()", Tango::ERR);
	}
	catch(...)
	{
		INFO_STREAM << __func__<<": NON Tango Exception sending command";
	}


	memset(read_str,0,256);
	size_t temp_len = resp.length();
	if(temp_len > 256)
	{
		WARN_STREAM << "Standa8SMC3::read_socket2(): read buffer overflow, " <<
			temp_len << " > " <<  256 << endl;
		temp_len = 256;
	}
	stringstream temp;
	for (int i = 0; i < temp_len; i++)
	{
		read_str[i] = resp[i];
		temp << showbase << hex << (int)resp[i] << " ";
	}
	DEBUG_STREAM << "Standa8SMC3::read_socket2(): read=" << temp.str() << endl;
}

//+------------------------------------------------------------------
//
//	method:			Standa8SMC3::check_timeout
//
//	description:	method to execute "check_timeout"
//					Return -1 if timeout [msec.] exceeded
//
//-------------------------------------------------------------------
int Standa8SMC3::check_timeout(timeval time, int timeout)
{
	struct timeval timenow;
	gettimeofday(&timenow,NULL);
	
	if ((timenow.tv_sec -	time.tv_sec) > (timeout/1000)) {
		DEBUG_STREAM << "Im540::check_timeout(): TIMEOUT! sec diff=" << (timenow.tv_sec - time.tv_sec) << " > timeout=" << (timeout/1000) << endl;
		return -1;
	}	
	if (((timenow.tv_sec -	time.tv_sec)*1000 - 
			(int)((float)(timenow.tv_usec -	time.tv_usec)/1000)) > timeout) {		
		return -1;
	}
	return 0;
	
}	/* end check_timeout() */	


//+------------------------------------------------------------------
//
//	method:			Standa8SMC3::abort_sleep
//
//	description:	Resume from sleep if abort_flag set (sec.)
//
//-------------------------------------------------------------------
void Standa8SMC3::abort_sleep(double time) 
{		
	
	for (int i = 0; i < (time/0.1); i++) {
		if (abortflag)
			break;
		omni_thread::sleep(0,100000000);
	}

}





//+------------------------------------------------------------------
/**
 *	method:	Standa8SMC3::forward
 *
 *	description:	method to execute "Forward"
 *	move forward for argin offset
 *
 * @param	argin	relative movement in user units
 *
 */
//+------------------------------------------------------------------
void Standa8SMC3::forward(Tango::DevDouble argin)
{
	DEBUG_STREAM << "Standa8SMC3::forward(): entering... !" << endl;

	//	Add your own code to control device here
	string resp;
	double steps_tmp;
	unsigned int steps_num;
	char steps_str[32];
	position_moving = 0;
	direction = 1;
	try {
		if(get_state() != Tango::ON)
			motor_on();
	} catch(...)
	{
		INFO_STREAM <<__func__<< ": exception in motor_on!()" << endl;
	}
	if(get_state() != Tango::ON)
	{
		try {
			if(get_state() != Tango::ON)
				motor_on();
		} catch(...)
		{
			INFO_STREAM <<__func__<< ": exception in motor_on!()" << endl;
		}
	}
	if(get_state() != Tango::ON)
	{
		Tango::Except::throw_exception(
			(const char *) "Command failed",
			(const char *) "Error switching Motor ON",
			(const char *) "Standa8SMC3::forward()", Tango::ERR);	
	}
	/*if(argin < -620 || argin > 620)
		Tango::Except::throw_exception(
			(const char *) "Command failed",
			(const char *) "Argin out of limits [-620,620]",
			(const char *) "Standa8SMC3::forward()", Tango::ERR);*/				
	abort_sleep((double(wr_sleep))/1000);	
	if(WriteReadStanda(SET_ROTATION_WAY,positive_rotation_way,resp, true))
		Tango::Except::throw_exception(
			(const char *) "Command failed",
			resp,
			(const char *) "Standa8SMC3::forward()", Tango::ERR);	
	DEBUG_STREAM << "Standa8SMC3::forward(): write = " << SET_ROTATION_WAY << " read = "<< resp << endl;
	abort_sleep((double(wr_sleep))/1000);
	steps_tmp = (argin * positionRatio * (double)step_factor);  //* 1 or * 2 or * 4 or * 8 (microsteps)
	steps_num = (unsigned int)steps_tmp;
/*	if(position_valid)
	{
		position_tmp += (steps_tmp / (double)step_factor);
		if(position_tmp > (360 * positionRatio))
			position_tmp -= (360 * positionRatio);
		else if(position_tmp < (-360 * positionRatio))
			position_tmp += (360 * positionRatio);
		DEBUG_STREAM << "Standa8SMC3::forward(): changing position_tmp to = " << position_tmp << endl;
	}*/
	sprintf(steps_str,"%6.6X",steps_num);
	DEBUG_STREAM << "Standa8SMC3::forward(): going to move steps_num = " << steps_num << " steps_str = "<< steps_str << " argin=" << argin << " position_tmp="<< position_tmp << endl;
	if(steps_num == 0)
		Tango::Except::throw_exception(
			(const char *) "Command failed",
			(const char *) "Argin value under minimum resolution",
			(const char *) "Standa8SMC3::forward()", Tango::ERR);
	if(WriteReadStanda(SET_STEP_NUMBER,steps_str,resp, true))
		Tango::Except::throw_exception(
			(const char *) "Command failed",
			resp,
			(const char *) "Standa8SMC3::forward()", Tango::ERR);	
	DEBUG_STREAM << "Standa8SMC3::forward(): write = " << SET_STEP_NUMBER << " read = "<< resp << endl;
	readloop->mutex_sleep->lock();
	readloop->pausesec_dfl = (int)(((double)th_sleep)/1000);					//poll status fast when not moving
	readloop->pausenano_dfl = ((th_sleep)-readloop->pausesec_dfl*1000)*1000000;
	moving = true;
	set_state(Tango::MOVING);		//Test	
	set_status("Motor MOVING");		//Test
	readloop->mutex_sleep->unlock();
	abort_sleep((double(wr_sleep))/1000);
	if(WriteReadStanda(CMD_MOVE,"",resp, true))
		Tango::Except::throw_exception(
			(const char *) "Command failed",
			resp,
			(const char *) "Standa8SMC3::forward()", Tango::ERR);	
	DEBUG_STREAM << "Standa8SMC3::forward(): write = " << CMD_MOVE << " read = "<< resp << endl;
	DEBUG_STREAM << "Standa8SMC3::forward() exiting with position_tmp= "<<position_tmp<<endl;	

}

//+------------------------------------------------------------------
/**
 *	method:	Standa8SMC3::backward
 *
 *	description:	method to execute "Backward"
 *	move backward for argin offset
 *
 * @param	argin	relative movement in user units
 *
 */
//+------------------------------------------------------------------
void Standa8SMC3::backward(Tango::DevDouble argin)
{
	DEBUG_STREAM << "Standa8SMC3::backward(): entering... !" << endl;

	//	Add your own code to control device here
	string resp;
	double steps_tmp;
	unsigned int steps_num;
	char steps_str[32];
	position_moving = 0;
	direction = -1;
	try {
		if(get_state() != Tango::ON)
			motor_on();
	} catch(...)
	{
		INFO_STREAM <<__func__<< ": exception in motor_on!()" << endl;
	}
	if(get_state() != Tango::ON)
	{
		try {
			if(get_state() != Tango::ON)
				motor_on();
		} catch(...)
		{
			INFO_STREAM <<__func__<< ": exception in motor_on!()" << endl;
		}
	}
	//TODO: control input value
	if(get_state() != Tango::ON)
		Tango::Except::throw_exception(
			(const char *) "Command failed",
			(const char *) "Error switching Motor ON",
			(const char *) "Standa8SMC3::backward()", Tango::ERR);
	/*if(argin < -620 || argin > 620)
		Tango::Except::throw_exception(
			(const char *) "Command failed",
			(const char *) "Argin out of limits [-620,620]",
			(const char *) "Standa8SMC3::backward()", Tango::ERR);*/			
	abort_sleep((double(wr_sleep))/1000);	
	if(WriteReadStanda(SET_ROTATION_WAY,negative_rotation_way,resp, true))
		Tango::Except::throw_exception(
			(const char *) "Command failed",
			resp,
			(const char *) "Standa8SMC3::backward()", Tango::ERR);	
	DEBUG_STREAM << "Standa8SMC3::backward(): write = " << SET_ROTATION_WAY << " read = "<< resp << endl;
	abort_sleep((double(wr_sleep))/1000);
	steps_tmp = (argin * positionRatio * (double)step_factor);  //* 1 or * 2 or * 4 or * 8 (microsteps)
	steps_num = (unsigned int)steps_tmp;
/*	if(position_valid)
	{
		position_tmp -= (steps_tmp / (double)step_factor);
		if(position_tmp > (360 * positionRatio))
			position_tmp -= (360 * positionRatio);
		else if(position_tmp < (-360 * positionRatio))
			position_tmp += (360 * positionRatio);		
		DEBUG_STREAM << "Standa8SMC3::backward(): changing position_tmp to = " << position_tmp << endl;
	}*/
	sprintf(steps_str,"%6.6X",steps_num);
	DEBUG_STREAM << "Standa8SMC3::backward(): going to move steps_num = " << steps_num << " steps_str = "<< steps_str << " argin=" << argin << " position_tmp="<< position_tmp << endl;
	if(steps_num == 0)
		Tango::Except::throw_exception(
			(const char *) "Command failed",
			(const char *) "Argin value under minimum resolution",
			(const char *) "Standa8SMC3::backward()", Tango::ERR);	
	if(WriteReadStanda(SET_STEP_NUMBER,steps_str,resp, true))
		Tango::Except::throw_exception(
			(const char *) "Command failed",
			resp,
			(const char *) "Standa8SMC3::backward()", Tango::ERR);	
	DEBUG_STREAM << "Standa8SMC3::backward(): write = " << SET_STEP_NUMBER << " read = "<< resp << endl;
	readloop->mutex_sleep->lock();
	readloop->pausesec_dfl = (int)(((double)th_sleep)/1000);					//poll status fast when not moving
	readloop->pausenano_dfl = ((th_sleep)-readloop->pausesec_dfl*1000)*1000000;
	moving = true;
	set_state(Tango::MOVING);		//Test	
	set_status("Motor MOVING");		//Test
	readloop->mutex_sleep->unlock();
	abort_sleep((double(wr_sleep))/1000);
	if(WriteReadStanda(CMD_MOVE,"",resp, true))
		Tango::Except::throw_exception(
			(const char *) "Command failed",
			resp,
			(const char *) "Standa8SMC3::backward()", Tango::ERR);	
	DEBUG_STREAM << "Standa8SMC3::backward(): write = " << CMD_MOVE << " read = "<< resp << endl;
	DEBUG_STREAM << "Standa8SMC3::backward() exiting with position_tmp= "<<position_tmp<<endl;

}

//+------------------------------------------------------------------
/**
 *	method:	Standa8SMC3::go_home
 *
 *	description:	method to execute "GoHome"
 *	move to home position (if defined)
 *
 *
 */
//+------------------------------------------------------------------
void Standa8SMC3::go_home()
{
	DEBUG_STREAM << "Standa8SMC3::go_home(): entering... !" << endl;
	//	Add your own code to control device here
	double val;	
	try {
		if(get_state() != Tango::ON)
			motor_on();
	} catch(...)
	{
		INFO_STREAM <<__func__<< ": exception in motor_on!()" << endl;
	}
	if(get_state() != Tango::ON)
	{
		try {
			if(get_state() != Tango::ON)
				motor_on();
		} catch(...)
		{
			INFO_STREAM <<__func__<< ": exception in motor_on!()" << endl;
		}
	}
	if(get_state() != Tango::ON)
		Tango::Except::throw_exception(
			(const char *) "Command failed",
			(const char *) "Error switching Motor ON",
			(const char *) "Standa8SMC3::go_home()", Tango::ERR);
#if 0		
	if(position_valid)
	{
/*		string resp;			//cannot go to exactly position 0 changing accuracy
		long val_acc = 1;
		char temp_c[32];
		sprintf(temp_c,"%d",val_acc);
		abort_sleep((double(wr_sleep))/1000);
		if(WriteReadStanda(SET_MICRO_STEP_CONF,temp_c,resp, true))
			WARN_STREAM << "Standa8SMC3::go_home(): exception writing micro step conf: '"<< resp << "'" << endl;		
		step_factor = 1;*/
		val = (position_tmp + position_moving) / (double)positionRatio;
		if(val > 0)
		{
			backward(val);
		}
		else if(val < 0)
		{
			forward(-val);
		}		
		else
		{
			INFO_STREAM << "Standa8SMC3::go_home(): already in home position!!!" << endl;
		}
	}
	else
		Tango::Except::throw_exception(
			(const char *) "Command failed",
			(const char *) "Home position has not been set!",
			(const char *) "Standa8SMC3::go_home()", Tango::ERR);	
#endif
	//first enable limit switches
	string resp;
	if(WriteReadStanda(ENABLE_SWITCH,"1",resp, true))
		WARN_STREAM << "Standa8SMC3::init_device(): exception enabling left limit switch: '"<< resp << "'" << endl;	
	abort_sleep((double(wr_sleep))/1000);
	if(WriteReadStanda(ENABLE_SWITCH,"2",resp, true))
		WARN_STREAM << "Standa8SMC3::init_device(): exception enabling right limit switch: '"<< resp << "'" << endl;

	find_edge(homing_direction);
}

//+------------------------------------------------------------------
/**
 *	method:	Standa8SMC3::set_home
 *
 *	description:	method to execute "SetHome"
 *	Set current position as home
 *
 *
 */
//+------------------------------------------------------------------
void Standa8SMC3::set_home()
{
	DEBUG_STREAM << "Standa8SMC3::set_home(): entering... !" << endl;

	//	Add your own code to control device here
	position_tmp = 0;
	position_moving = 0;
	position_valid = true;

}

//+------------------------------------------------------------------
/**
 *	method:	Standa8SMC3::go_to_position
 *
 *	description:	method to execute "GoToPosition"
 *	go to absolute position (if home defined)
 *
 * @param	argin	absolute position in user units
 *
 */
//+------------------------------------------------------------------
void Standa8SMC3::go_to_position(Tango::DevDouble argin)
{
	DEBUG_STREAM << "Standa8SMC3::go_to_position(): entering... !" << endl;

	//	Add your own code to control device here
	double val;
	double actual_pos;
	try {
		if(get_state() != Tango::ON)
			motor_on();
	} catch(...)
	{
		INFO_STREAM <<__func__<< ": exception in motor_on!()" << endl;
	}
	if(get_state() != Tango::ON)
	{
		try {
			if(get_state() != Tango::ON)
				motor_on();
		} catch(...)
		{
			INFO_STREAM <<__func__<< ": exception in motor_on!()" << endl;
		}
	}
	if(get_state() != Tango::ON)
		Tango::Except::throw_exception(
			(const char *) "Command failed",
			(const char *) "Error switching Motor ON",
			(const char *) "Standa8SMC3::go_to_position()", Tango::ERR);
	if(argin < axisBackwardLimit || argin > axisForwardLimit)
	{
		stringstream tmp;
		tmp << "Argin out of limits ["<<axisBackwardLimit<<","<<axisForwardLimit<<"]" << ends;
		Tango::Except::throw_exception(
			(const char *) "Command failed",
			(const char *) tmp.str().c_str(),
			(const char *) "Standa8SMC3::go_to_position()", Tango::ERR);			
	}	
	if(position_valid)
	{
		actual_pos = (position_tmp + position_moving) / (double)positionRatio;
		val = actual_pos - argin;
		if(val > 0)
		{
			backward(val);
		}
		else if(val < 0)
		{
			forward(-val);
		}		
		else
		{
			INFO_STREAM << "Standa8SMC3::go_to_position(): already in requested position="<< argin << endl ;
		}		
	}
	else
		Tango::Except::throw_exception(
			(const char *) "Command failed",
			(const char *) "Home position has not been set!",
			(const char *) "Standa8SMC3::go_to_position()", Tango::ERR);
}









//+------------------------------------------------------------------
/**
 *	method:	Standa8SMC3::find_edge
 *
 *	description:	method to execute "FindEdge"
 *	Move to left (argin=0) or right (argin=1) limit switch
 *
 * @param	argin	0=find left limit switch 1=find right limit switch
 *
 */
//+------------------------------------------------------------------
void Standa8SMC3::find_edge(Tango::DevLong argin)
{
	DEBUG_STREAM << "Standa8SMC3::find_edge(): entering... !" << endl;

	//	Add your own code to control device here
	string resp;
	abort_sleep((double(wr_sleep))/1000);
	char arg[64];
	sprintf(arg, "%d", argin);
	if(WriteReadStanda(CMD_RESET,arg,resp, true))		//reset left
		Tango::Except::throw_exception(
			(const char *) "Command failed",
			"exception resetting to limit switch: " + resp,
			(const char *) "Standa8SMC3::find_edge()", Tango::ERR);	
		//WARN_STREAM << "Standa8SMC3::init_device(): exception resetting to left limit switch: '"<< resp << "'" << endl;
	DEBUG_STREAM << "Standa8SMC3::find_edge(): write = " << CMD_RESET << " read = "<< resp << endl;
	homing = true;
	old_state = device_state;
	old_status = device_status;
	set_state(Tango::MOVING);	
	set_status("Motor HOMING");

}


}	//	namespace
